void Main() {
	// standard edition check
	if (Permissions::CanSubscribeToStandard()) {
		UI::ShowNotification(Icons::ExclamationTriangle + "  Standard edition required!", "As you can play custom maps with this plugin, the Standard edition of Trackmania is required.", vec4(0.6, 0.6, 1.0, 0.27), 30000);
		warn("As you can play custom maps with this plugin, the Standard edition of Trackmania is required.");
		return;
	}

	if (Setting_TMDataLastSave != Meta::ExecutingPlugin().Version) {
		startnew(CopyModeFiles);
	}
	startnew(TwitchLoop);
	startnew(InputsLoop);
}

void RenderInterface() {
	// standard edition check
	if (Permissions::CanSubscribeToStandard()) return;

	if (!Setting_WindowVisible) return;

	if (Setting_MoveOverlay) {
		Locator::Render("Overlay", Setting_OverlayPos, Setting_OverlaySize);
		Setting_OverlayPos = Locator::GetPos();
		Setting_OverlaySize = Locator::GetSize();
	}

	if (UI::Begin("\\$f50" + Icons::Twitch + "\\$fff  Twitch Plays Trackmania \\$555 by Nsgr###TwitchPlays", Setting_WindowVisible, UI::WindowFlags::NoScrollbar | UI::WindowFlags(1 << 21) | UI::WindowFlags::NoCollapse)) {
		g_twitchPlays.m_ui.Render();
		UI::End();
	}
}

void RenderMenu() {
	// standard edition check
	if (Permissions::CanSubscribeToStandard()) return;

	if (UI::MenuItem("\\$f50" + Icons::Twitch + "\\$fff  Twitch Plays", "", Setting_WindowVisible)) {
		Setting_WindowVisible = !Setting_WindowVisible;
	}
}

void CopyModeFiles() {
	string[] files = CSV::FromFileSource("data/Files.csv")[0];
	for (uint i = 0; i < files.Length; i++) {
		string sourceFile = IO::FileSource("ModeFiles/" + files[i]).ReadToEnd();

		// Need to create the folder before writing the file.
		string fileToWrite = IO::FromUserGameFolder(files[i]);
		// Cut off the file part to only create a folder until there.
		string fileNameOfPath = fileToWrite.Split("/")[fileToWrite.Split("/").Length - 1];
		string folderToCreate = fileToWrite.SubStr(0, fileToWrite.Length - fileNameOfPath.Length);
		IO::CreateFolder(folderToCreate);
		// Write file.
		IO::File destFile(fileToWrite, IO::FileMode::Write);
		destFile.Write(sourceFile);
		destFile.Close();
	}
	// If no errors happened, we land here
	Setting_TMDataLastSave = Meta::ExecutingPlugin().Version;
}

void StartRun() {
	MwFastBuffer<wstring> mapList;
	auto titleControl = cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI;
	for (uint i = 0; i < g_twitchPlays.m_selectedMaps.Length; i++) {
	    mapList.Add(g_twitchPlays.m_selectedMaps[i].m_path);
	}
	titleControl.PlayMapList(mapList, "TrackMania/TwitchPlays", "");
	if (titleControl.LatestResult != CGameManiaTitleControlScriptAPI::EResult::Success) {
		string completeResult = "";
		for (uint i = 0; i < titleControl.CustomResultData.Length; i++) {
			completeResult += titleControl.CustomResultData[i] + "\n";
		}
		error(tostring(titleControl.LatestResult) + "\n" + titleControl.CustomResultType + "\n" + completeResult);
		UI::ShowNotification(tostring(titleControl.LatestResult), titleControl.CustomResultType + "\n" + completeResult);
		if (titleControl.LatestResult == CGameManiaTitleControlScriptAPI::EResult::Error_DataMgr) {
			UI::ShowNotification(Icons::Times + "  Bruh", "\\$f66Selecting an unvalidated / incompatible map for local play freezes the Game title control. You will need to reboot the game.", vec4(1.0, 0.0, 0.0, 0.27), 60000);
		}
		g_twitchPlays.m_isActive = false;
		g_twitchPlays.m_ui.m_tabState = TabState::MapOrder;
	} else {
		g_twitchPlays.m_isActive = true;
		g_twitchPlays.m_ui.m_tabState = TabState::Running;
	}
}