int g_nvgFont = nvg::LoadFont("DroidSans.ttf", true);
Overlay@ g_overlay = Overlay();

const vec4 OVERLAY_STEERING_COLOR = vec4(0.8, 0.4, 0.0, 1.0);
const vec4 OVERLAY_ACCEL_COLOR = vec4(0.0, 0.8, 0.0, 1.0);
const vec4 OVERLAY_BRAKE_COLOR = vec4(0.8, 0.0, 0.0, 1.0);

class Overlay {
	uint64 m_lastLeft;
	string m_lastLeftUser;

	uint64 m_lastRight;
	string m_lastRightUser;

	uint64 m_lastForward;
	string m_lastForwardUser;

	uint64 m_lastBackward;
	string m_lastBackwardUser;

	uint64 m_lastRespawn;
	string m_lastRespawnUser;
}

void Render() {
	// standard edition check
	if (Permissions::CanSubscribeToStandard()) return;

	if (g_twitchPlays.m_isActive) {
		
		vec2 screenSize = vec2(Draw::GetWidth(), Draw::GetHeight());
		vec2 pos = Setting_OverlayPos * (screenSize - Setting_OverlaySize);
		nvg::Translate(pos.x, pos.y);
		pos = vec2(0,0);

		nvg::BeginPath();
		nvg::FontFace(g_nvgFont);

		// Overlay
		// bar |-| full size, text on side 0.2 username, text inside is command, text center is count l / r
		// square |-| then bar |-|, 

		vec2 tileSize = vec2(Setting_OverlaySize.x, Setting_OverlaySize.y * 0.4);

		RenderSteeringBar(pos, tileSize, g_inputs.m_inputSteer, Setting_LeftInput, tostring(g_inputs.m_InputsLeft.Length) + " / " + tostring(g_inputs.m_InputsRight.Length), Setting_RightInput);
		RenderNames(pos, tileSize, g_overlay.m_lastLeft, g_overlay.m_lastLeftUser, g_overlay.m_lastRight, g_overlay.m_lastRightUser);

		pos = vec2(pos.x, pos.y + Setting_OverlaySize.y * 0.5);

		RenderAccelBar(pos, tileSize, g_inputs.m_inputAccelerate, g_inputs.m_inputBrake, Setting_ForwardInput, tostring(g_inputs.m_InputsForward.Length) + " / " + tostring(g_inputs.m_InputsBackward.Length), Setting_BrakeInput);
		RenderNames(pos, tileSize, g_overlay.m_lastForward, g_overlay.m_lastForwardUser, g_overlay.m_lastBackward, g_overlay.m_lastBackwardUser);

		pos = vec2(pos.x, pos.y + Setting_OverlaySize.y * 0.5);

		RenderRespawnText(pos, tileSize, g_overlay.m_lastRespawn, g_overlay.m_lastRespawnUser);

		nvg::ResetTransform();
	}
}

float CalculateFade(uint64 eventTime) {
	if (eventTime > Time::Now) return 1.0;
	uint64 vanishTimestamp = Time::Now - int(Setting_OverlayNameFadeOut * 1000);
	if (eventTime > vanishTimestamp) return (eventTime - vanishTimestamp) / (Setting_OverlayNameFadeOut * 1000);
	return 0.0;
}


void RenderNames(vec2 pos, vec2 size, uint64 leftTimestamp, const string &in leftText, uint64 rightTimestamp, const string &in rightText) {
	float textPadding = 0.25 * size.y;
	nvg::FontFace(g_nvgFont);

	nvg::BeginPath();
	nvg::FontSize(size.y * 0.5);

	nvg::TextAlign(nvg::Align::Right | nvg::Align::Middle);
	nvg::FillColor(vec4(1,1,1,CalculateFade(leftTimestamp)));
	nvg::Text(pos.x - textPadding, pos.y + size.y * 0.5, leftText);

	nvg::TextAlign(nvg::Align::Left | nvg::Align::Middle);
	nvg::FillColor(vec4(1,1,1,CalculateFade(rightTimestamp)));
	nvg::Text(pos.x + size.x + textPadding, pos.y + size.y * 0.5, rightText);
}

void RenderSteeringBar(vec2 pos, vec2 size, float steering, const string &in left, const string &in center, const string &in right) {
	float textPadding = 0.25 * size.y;
	nvg::FontFace(g_nvgFont);

	// Bar background
	nvg::BeginPath();
	nvg::FillColor(vec4(0,0,0,0.5));
	nvg::Rect(pos.x, pos.y, size.x, size.y);
	nvg::Fill();

	// bar foreground (only drawn when enough steering exists)
	if (steering > 0.01 || steering < -0.01) {
		nvg::BeginPath();
		nvg::FillColor(vec4(OVERLAY_STEERING_COLOR.x, OVERLAY_STEERING_COLOR.y, OVERLAY_STEERING_COLOR.z, OVERLAY_STEERING_COLOR.w));
		nvg::Rect(pos.x + size.x * 0.5, pos.y, steering * size.x * 0.5, pos.y + size.y);
		nvg::Fill();
	}

	// texts
	nvg::BeginPath();
	nvg::FillColor(vec4(1,1,1,1));
	nvg::FontSize(size.y * 0.5);
	if (left != "") {
		nvg::TextAlign(nvg::Align::Left | nvg::Align::Middle);
		nvg::Text(pos.x + textPadding, pos.y + size.y * 0.5, left);
	}

	if (center != "") {
		nvg::TextAlign(nvg::Align::Center | nvg::Align::Middle);
		nvg::Text(pos.x + size.x * 0.5, pos.y + size.y * 0.5, center);
	}
	
	if (right != "") {
		nvg::TextAlign(nvg::Align::Right | nvg::Align::Middle);
		nvg::Text(pos.x + size.x - textPadding, pos.y + size.y * 0.5, right);
	}
}

void RenderAccelBar(vec2 pos, vec2 size, bool accel, float brake, const string &in left, const string &in center, const string &in right) {
	float textPadding = 0.25 * size.y;
	nvg::FontFace(g_nvgFont);

	// assuming size.y < size.x, but who would resize it that way anyway Clueless

	// Bar background
	// Accel square
	nvg::BeginPath();
	nvg::FillColor(vec4(0,0,0, 0.5));
	nvg::Rect(pos.x, pos.y, size.y, size.y);
	nvg::Fill();

	// brake bar
	nvg::BeginPath();
	nvg::FillColor(vec4(0,0,0, 0.5));
	nvg::Rect(pos.x + size.y + (0.1 * size.x), pos.y, size.x - (size.y + (0.1 * size.x)), size.y);
	nvg::Fill();

	// Bar foreground
	if (accel) {
		nvg::BeginPath();
		nvg::FillColor(OVERLAY_ACCEL_COLOR);
		nvg::Rect(pos.x, pos.y, size.y, size.y);
		nvg::Fill();
	}

	if (brake > 0.01) {
		nvg::BeginPath();
		nvg::FillColor(OVERLAY_BRAKE_COLOR);
		nvg::Rect(pos.x + size.y + (0.1 * size.x), pos.y, (size.x - (size.y + (0.1 * size.x))) * brake, size.y);
		nvg::Fill();
	}

	// texts
	nvg::BeginPath();
	nvg::FillColor(vec4(1,1,1,1));
	nvg::FontSize(size.y * 0.5);
	if (left != "") {
		nvg::TextAlign(nvg::Align::Left | nvg::Align::Middle);
		nvg::Text(pos.x + textPadding, pos.y + size.y * 0.5, left);
	}

	if (center != "") {
		nvg::TextAlign(nvg::Align::Center | nvg::Align::Middle);
		nvg::Text(pos.x + size.x * 0.5, pos.y + size.y * 0.5, center);
	}
	
	if (right != "") {
		nvg::TextAlign(nvg::Align::Right | nvg::Align::Middle);
		nvg::Text(pos.x + size.x - textPadding, pos.y + size.y * 0.5, right);
	}
}

void RenderRespawnText(vec2 pos, vec2 size, uint64 respawnTimestamp, const string &in respawnUser) {
	nvg::FontFace(g_nvgFont);

	nvg::BeginPath();
	nvg::FillColor(vec4(1,1,1,CalculateFade(respawnTimestamp)));
	nvg::FontSize(size.y * 0.5);
	nvg::TextAlign(nvg::Align::Center | nvg::Align::Middle);

	nvg::Text(pos.x + size.x * 0.5, pos.y + size.y * 0.5, "rs: " + respawnUser);
}