int g_pluginId;

enum ChatRole {
	Broadcaster = 4,
	Moderator = 3,
	VIP = 2,
	Subscriber = 1,
	Everyone = 0
}

ChatRole GetChatRole(Twitch::Message@ message) {
	if (message.m_tags.Exists("badges")) {
		string badges = string(message.m_tags["badges"]);
		if (badges.Contains("broadcaster/")) return ChatRole::Broadcaster;
		if (badges.Contains("moderator/") || badges.Contains("global_mod/")) return ChatRole::Moderator;
		if (badges.Contains("vip/")) return ChatRole::VIP;
		if (badges.Contains("subscriber/") || badges.Contains("founder/")) return ChatRole::Subscriber;
	}
	return ChatRole::Everyone;
}

void TwitchLoop() {
	while (!g_twitchPlays.m_isActive) yield();
	while (true) {
		// Standard registration / fetch
		if (g_pluginId == 0) g_pluginId = Twitch::Register(Twitch::MessageType::ChatMessage);
		array<Twitch::Message@> newMessages = Twitch::Fetch(g_pluginId);

		// Let old messages expire
		// if youre not me do not look at this
		for (uint i = 0; i < g_inputs.m_InputsLeft.Length; i++) {
			if (Time::Now > g_inputs.m_InputsLeft[i]) {
				g_inputs.m_InputsLeft.RemoveAt(i);
				i--;
			}
		}
		for (uint i = 0; i < g_inputs.m_InputsRight.Length; i++) {
			if (Time::Now > g_inputs.m_InputsRight[i]) {
				g_inputs.m_InputsRight.RemoveAt(i);
				i--;
			}
		}
		for (uint i = 0; i < g_inputs.m_InputsBackward.Length; i++) {
			if (Time::Now > g_inputs.m_InputsBackward[i]) {
				g_inputs.m_InputsBackward.RemoveAt(i);
				i--;
			}
		}
		for (uint i = 0; i < g_inputs.m_InputsForward.Length; i++) {
			if (Time::Now > g_inputs.m_InputsForward[i]) {
				g_inputs.m_InputsForward.RemoveAt(i);
				i--;
			}
		}

		// Add new messages
		if (g_twitchPlays.m_isActive) {
			for (uint i = 0; i < newMessages.Length; i++) {
				ProcessMessage(newMessages[i]);
			}
		}

		yield();
	}
}

void ProcessMessage(Twitch::Message@ message) {
	if (Setting_MultiInput) {
		if (message.m_text.Contains(Setting_LeftInput)) {
			g_inputs.m_InputsLeft.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastLeft = Time::Now;
			g_overlay.m_lastLeftUser = message.m_username;
		}
		if (message.m_text.Contains(Setting_RightInput)) {
			g_inputs.m_InputsRight.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastRight = Time::Now;
			g_overlay.m_lastRightUser = message.m_username;
		}
		if (message.m_text.Contains(Setting_BrakeInput)) {
			g_inputs.m_InputsBackward.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastBackward = Time::Now;
			g_overlay.m_lastBackwardUser = message.m_username;
		}
		if (message.m_text.Contains(Setting_ForwardInput)) {
			g_inputs.m_InputsForward.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastForward = Time::Now;
			g_overlay.m_lastForwardUser = message.m_username;
		}
		if (message.m_text.Contains(Setting_RespawnInput) && GetChatRole(message) >= Setting_RespawnRequiredRole) {
			g_overlay.m_lastRespawnUser = message.m_username;
			g_inputs.m_InputRespawn = true;
		}
	} else {
		if (message.m_text == Setting_LeftInput) {
			g_inputs.m_InputsLeft.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastLeft = Time::Now;
			g_overlay.m_lastLeftUser = message.m_username;
		} else if (message.m_text == Setting_RightInput) {
			g_inputs.m_InputsRight.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastRight = Time::Now;
			g_overlay.m_lastRightUser = message.m_username;
		} else if (message.m_text == Setting_BrakeInput) {
			g_inputs.m_InputsBackward.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastBackward = Time::Now;
			g_overlay.m_lastBackwardUser = message.m_username;
		} else if (message.m_text == Setting_ForwardInput) {
			g_inputs.m_InputsForward.InsertLast(Time::Now + uint(Setting_InputEffectDuration * 1000));
			g_overlay.m_lastForward = Time::Now;
			g_overlay.m_lastForwardUser = message.m_username;
		} else if (message.m_text == Setting_RespawnInput && GetChatRole(message) >= Setting_RespawnRequiredRole) {
			g_overlay.m_lastRespawnUser = message.m_username;
			g_inputs.m_InputRespawn = true;
		}
	}
}

void OnDisabled() {
	if (g_pluginId > 0) Twitch::Unregister(g_pluginId);
	g_pluginId = 0;
}

void OnDestroyed() {
	if (g_pluginId > 0) Twitch::Unregister(g_pluginId);
	g_pluginId = 0;
}

void OnSettingsChanged() {
	g_inputs.m_InputsLeft.RemoveRange(0, g_inputs.m_InputsLeft.Length);
	g_inputs.m_InputsRight.RemoveRange(0, g_inputs.m_InputsRight.Length);
	g_inputs.m_InputsBackward.RemoveRange(0, g_inputs.m_InputsBackward.Length);
	g_inputs.m_InputsForward.RemoveRange(0, g_inputs.m_InputsForward.Length);
}