UI::Font@ g_uiFontBold = UI::LoadFont("DroidSans-Bold.ttf");

class UI {
	TabState m_tabState;

	void Render() {
		switch (m_tabState) {
			case TabState::MainMenu :
				RenderMainMenu();
				break;
			case TabState::MapSelection :
				RenderMapSelection();
				break;
			case TabState::MapOrder : 
				RenderMapOrder();
				break;
			case TabState::Running : 
				RenderInGame();
				break;
			default: 
				UI::Text("\\$faaOops! I don't know what to render here.");
		}
	}

	// Render "main menu"
	void RenderMainMenu() {
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5) - 25));
		CenterLabel("Twitch Plays Trackmania " + Meta::ExecutingPlugin().Version);
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5)));
		if (CenterButton(Icons::Play + "  Start playing")) {
			startnew(RefreshIndexedMaps);
			m_tabState = TabState::MapSelection;
		}
	}

	// Render map selection
	void RenderMapSelection() {
		vec2 tabSize = vec2(UI::GetWindowSize().x - 16, UI::GetWindowSize().y - UI::GetCursorPos().y - 36);
		UI::BeginChild("MapSelection", tabSize);
		BoldLabel("\\$faaSelecting incompatible maps may force you to restart the game.");
		if (g_twitchPlays.m_indexedMapRoot !is null) {
			UI::PushStyleColor(UI::Col::FrameBg, vec4(0.5,0.5,0.5,0.1));
			UI::PushStyleColor(UI::Col::FrameBgHovered, vec4(1.0,1.0,1.0,0.1));
			UI::PushStyleColor(UI::Col::FrameBgActive, vec4(0.75,0.75,0.75,0.1));
			UI::PushStyleColor(UI::Col::CheckMark, vec4(0.357,0.888,0.922, 1.0));
			RenderFolder(g_twitchPlays.m_indexedMapRoot);
			UI::PopStyleColor(4);
		}
		UI::EndChild();
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, UI::GetCursorPos().y - 6));
		UI::Separator();
		vec2 pos_orig = UI::GetCursorPos();
		if (UI::Button(Icons::AngleDoubleLeft + "  Menu")) {
			m_tabState = TabState::MainMenu;
		}

		UI::SetCursorPos(vec2(UI::GetWindowSize().x * 0.5 - Draw::MeasureString(Icons::Redo + "       maps selected" + g_twitchPlays.m_selectedMaps.Length).x * 0.5, pos_orig.y));
		if (UI::Button(Icons::Redo)) {
			startnew(RefreshIndexedMaps);
		}
		UI::SameLine();
		UI::Text(g_twitchPlays.m_selectedMaps.Length + " maps selected");


		UI::SetCursorPos(vec2(UI::GetWindowSize().x - Draw::MeasureString("Map order        " + Icons::AngleDoubleRight).x, pos_orig.y));
		UI::BeginDisabled(g_twitchPlays.m_selectedMaps.Length < 1);
		if (UI::Button("Map order  " + Icons::AngleDoubleRight)) {
			m_tabState = TabState::MapOrder;
		}
		UI::EndDisabled();
	}

	// Render map order
	void RenderMapOrder() {
		vec2 tabSize = vec2(UI::GetWindowSize().x - 16, UI::GetWindowSize().y - UI::GetCursorPos().y - 36);
		UI::BeginChild("MapOrder", tabSize);
		for (uint i = 0; i < g_twitchPlays.m_selectedMaps.Length; i++) {
			UI::PushID(g_twitchPlays.m_selectedMaps[i]);
			if (UI::Button(Icons::AngleUp)) {
				Map@ selectedMap = g_twitchPlays.m_selectedMaps[i];
				g_twitchPlays.m_selectedMaps.RemoveAt(i);
				g_twitchPlays.m_selectedMaps.InsertAt(Math::Max(0, i-1), selectedMap);
			}
			UI::SameLine();
			if (UI::Button(Icons::AngleDown)) {
				Map@ selectedMap = g_twitchPlays.m_selectedMaps[i];
				g_twitchPlays.m_selectedMaps.RemoveAt(i);
				g_twitchPlays.m_selectedMaps.InsertAt(Math::Min(g_twitchPlays.m_selectedMaps.Length, i+1), selectedMap);
			}
			UI::SameLine();
			UI::Text(g_twitchPlays.m_selectedMaps[i].m_displayName);
			UI::PopID();
		}
		UI::EndChild();
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, UI::GetCursorPos().y - 6));
		UI::Separator();

		vec2 pos_orig = UI::GetCursorPos();
		if (UI::Button(Icons::AngleDoubleLeft + "  Map selection")) {
			m_tabState = TabState::MapSelection;
		}

		UI::BeginDisabled(!cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI.IsReady);

		UI::SetCursorPos(vec2(UI::GetWindowSize().x - Draw::MeasureString("Start        " + Icons::AngleDoubleRight).x, pos_orig.y));
		if (UI::Button("Start  " + Icons::AngleDoubleRight)) {
			StartRun();
		}

		if (UI::IsItemHovered() && !cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI.IsReady) {
			UI::BeginTooltip();
			UI::Text("You have to be in the main menu to start playing!");
			UI::EndTooltip();
		}
		UI::EndDisabled();
	}

	// Render while runnning poll
	void RenderInGame() {
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5) - 25));
		CenterLabel("Run in progress...");
		UI::SetCursorPos(vec2(UI::GetCursorPos().x, (UI::GetWindowSize().y * 0.5)));
		UI::PushStyleColor(UI::Col::Button, vec4(0.647,0.176,0.157, 1.0));
		UI::PushStyleColor(UI::Col::ButtonHovered, vec4(0.98,0.275,0.251, 1.0));
		UI::PushStyleColor(UI::Col::ButtonActive, vec4(0.98,0.027,0.059, 1.0));
		if (CenterButton(Icons::PowerOff + "  Stop run")) {
			g_twitchPlays.m_isActive = false;
			m_tabState = TabState::MainMenu;
		}
		UI::PopStyleColor(3);
	}


	// File selection

	void RenderFolder(Folder@ folder) {
		UI::PushID(folder.m_name);
		if (UI::Checkbox("\\$z", folder.m_selected) != folder.m_selected) {
			UpdateFolderSelection(folder, !folder.m_selected);
		}
		UI::SameLine();
		if (UI::TreeNode("\\$ff7" + Icons::FolderO + " \\$fff" + folder.m_name)) {
			for (uint i = 0; i < folder.m_folders.Length; i++) {
				UI::TreeAdvanceToLabelPos();
				RenderFolder(folder.m_folders[i]);
			}
			for (uint i = 0; i < folder.m_maps.Length; i++) {
				UI::TreeAdvanceToLabelPos();
				RenderMap(folder.m_maps[i]);
			}
			UI::TreePop();
		}
		UI::PopID();
	}

	void RenderMap(Map@ map) {
		bool mapSelected = FindMapSelectionIndex(map) >= 0;
		if (UI::Checkbox("\\$5f5" + Icons::FileO + " \\$fff" + map.m_displayName, mapSelected) != mapSelected) {
			UpdateMapSelection(map, !mapSelected);
		}
	}

	// General helpers
	void CenterLabel(const string &in Label) {
		vec2 pos_orig = UI::GetCursorPos();

		vec2 LabelSize = Draw::MeasureString(Label, g_uiFontBold);
		UI::SetCursorPos(vec2((UI::GetWindowSize().x - LabelSize.x) / 2, UI::GetCursorPos().y));
		BoldLabel(Label);

		UI::SetCursorPos(pos_orig);
	}

	void BoldLabel(const string &in Label) {
		UI::PushFont(g_uiFontBold);
		UI::Text(Label);
		UI::PopFont();
	}

	bool CenterButton(const string &in Label, bool bold = false) {
		if (bold) UI::PushFont(g_uiFontBold);
		vec2 pos_orig = UI::GetCursorPos();
		vec2 LabelSize = Draw::MeasureString(Label + "  ", bold ? g_uiFontBold : null);
		UI::SetCursorPos(vec2((UI::GetWindowSize().x - LabelSize.x) / 2, UI::GetCursorPos().y));
		bool result = UI::Button(Label);
		if (bold) UI::PopFont();
		return result;
	}
}

enum TabState {
	MainMenu,
	MapSelection,
	MapOrder,
	Running
}